@@ -52,6 +52,8 @@
 #include <errno.h>
 #include <sys/types.h>
 
+#include <full-write.h>
+
 #include "libguile/iselect.h"
 
 /* Some defines for Windows (native port, not Cygwin). */
@@ -266,6 +268,46 @@ SCM_DEFINE (scm_file_port_p, "file-port?", 1, 0, 0,
 #undef FUNC_NAME
 
 
+static SCM sys_file_port_name_canonicalization;
+SCM_SYMBOL (sym_relative, "relative");
+SCM_SYMBOL (sym_absolute, "absolute");
+
+static SCM
+fport_canonicalize_filename (SCM filename)
+{
+  SCM mode = scm_fluid_ref (sys_file_port_name_canonicalization);
+
+  if (!scm_is_string (filename))
+    {
+      return filename;
+    }
+  else if (scm_is_eq (mode, sym_relative))
+    {
+      SCM path, rel;
+
+      path = scm_variable_ref (scm_c_module_lookup (scm_the_root_module (),
+                                                    "%load-path"));
+      rel = scm_i_relativize_path (filename, path);
+
+      return scm_is_true (rel) ? rel : filename;
+    }
+  else if (scm_is_eq (mode, sym_absolute))
+    {
+      char *str, *canon;
+  
+      str = scm_to_locale_string (filename);
+      canon = canonicalize_file_name (str);
+      free (str);
+  
+      return canon ? scm_take_locale_string (canon) : filename;
+    }
+  else
+    {
+      return filename;
+    }
+}
+
+
 /* scm_open_file
  * Return a new port open on a given file.
  *
@@ -275,7 +317,7 @@ SCM_DEFINE (scm_file_port_p, "file-port?", 1, 0, 0,
  * Return the new port.
  */
 SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
-           (SCM filename, SCM mode),
+	    (SCM filename, SCM mode),
 	    "Open the file whose name is @var{filename}, and return a port\n"
 	    "representing that file.  The attributes of the port are\n"
 	    "determined by the @var{mode} string.  The way in which this is\n"
@@ -296,7 +338,10 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	    "The following additional characters can be appended:\n"
 	    "@table @samp\n"
 	    "@item b\n"
-	    "Open the underlying file in binary mode, if supported by the operating system. "
+	    "Open the underlying file in binary mode, if supported by the system.\n"
+	    "Also, open the file using the binary-compatible character encoding\n"
+	    "\"ISO-8859-1\", ignoring the port's encoding and the coding declaration\n"
+	    "at the top of the input file, if any.\n"
 	    "@item +\n"
 	    "Open the port for both input and output.  E.g., @code{r+}: open\n"
 	    "an existing file for both input and output.\n"
@@ -311,6 +356,11 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	    "Add line-buffering to the port.  The port output buffer will be\n"
 	    "automatically flushed whenever a newline character is written.\n"
 	    "@end table\n"
+	    "When the file is opened, this procedure will scan for a coding\n"
+	    "declaration@pxref{Character Encoding of Source Files}. If present\n"
+	    "will use that encoding for interpreting the file.  Otherwise, the\n"
+	    "port's encoding will be used.\n"
+	    "\n"
 	    "In theory we could create read/write ports which were buffered\n"
 	    "in one direction only.  However this isn't included in the\n"
 	    "current interfaces.  If a file cannot be opened with the access\n"
@@ -318,7 +368,7 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 #define FUNC_NAME s_scm_open_file
 {
   SCM port;
-  int fdes, flags = 0;
+  int fdes, flags = 0, use_encoding = 1;
   unsigned int retries;
   char *file, *md, *ptr;
 
@@ -353,6 +403,7 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	  flags = (flags & ~(O_RDONLY | O_WRONLY)) | O_RDWR;
 	  break;
 	case 'b':
+	  use_encoding = 0;
 #if defined (O_BINARY)
 	  flags |= O_BINARY;
 #endif
@@ -386,7 +437,26 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	}
     }
 
-  port = scm_i_fdes_to_port (fdes, scm_i_mode_bits (mode), filename);
+  /* Create a port from this file descriptor.  The port's encoding is initially
+     %default-port-encoding.  */
+  port = scm_i_fdes_to_port (fdes, scm_i_mode_bits (mode),
+                             fport_canonicalize_filename (filename));
+
+  if (use_encoding)
+    {
+      /* If this file has a coding declaration, use that as the port
+	 encoding.  */
+      if (SCM_INPUT_PORT_P (port))
+	{
+	  char *enc = scm_i_scan_for_encoding (port);
+	  if (enc != NULL)
+	    scm_i_set_port_encoding_x (port, enc);
+	}
+    }
+  else
+    /* If this is a binary file, use the binary-friendly ISO-8859-1
+       encoding.  */
+    scm_i_set_port_encoding_x (port, NULL);
 
   scm_dynwind_end ();
 
@@ -752,63 +822,18 @@ fport_write (SCM port, const void *data, size_t size)
 }
 #undef FUNC_NAME
 
-/* becomes 1 when process is exiting: normal exception handling won't
-   work by this time.  */
-extern int scm_i_terminating; 
-
 static void
 fport_flush (SCM port)
 {
+  size_t written;
   scm_t_port *pt = SCM_PTAB_ENTRY (port);
   scm_t_fport *fp = SCM_FSTREAM (port);
-  unsigned char *ptr = pt->write_buf;
-  long init_size = pt->write_pos - pt->write_buf;
-  long remaining = init_size;
+  size_t count = pt->write_pos - pt->write_buf;
 
-  while (remaining > 0)
-    {
-      long count;
+  written = full_write (fp->fdes, pt->write_buf, count);
+  if (written < count)
+    scm_syserror ("scm_flush");
 
-      SCM_SYSCALL (count = write (fp->fdes, ptr, remaining));
-      if (count < 0)
-	{
-	  /* error.  assume nothing was written this call, but
-	     fix up the buffer for any previous successful writes.  */
-	  long done = init_size - remaining;
-	      
-	  if (done > 0)
-	    {
-	      int i;
-
-	      for (i = 0; i < remaining; i++)
-		{
-		  *(pt->write_buf + i) = *(pt->write_buf + done + i);
-		}
-	      pt->write_pos = pt->write_buf + remaining;
-	    }
-	  if (scm_i_terminating)
-	    {
-	      const char *msg = "Error: could not flush file-descriptor ";
-	      char buf[11];
-
-	      full_write (2, msg, strlen (msg));
-	      sprintf (buf, "%d\n", fp->fdes);
-	      full_write (2, buf, strlen (buf));
-
-	      count = remaining;
-	    }
-	  else if (scm_gc_running_p)
-	    {
-	      /* silently ignore the error.  scm_error would abort if we
-		 called it now.  */
-	      count = remaining;
-	    }
-	  else
-	    scm_syserror ("fport_flush");
-	}
-      ptr += count;
-      remaining -= count;
-    }
   pt->write_pos = pt->write_buf;
   pt->rw_active = SCM_PORT_NEITHER;
 }
@@ -894,6 +919,10 @@ scm_init_fports ()
   scm_c_define ("_IOLBF", scm_from_int (_IOLBF));
   scm_c_define ("_IONBF", scm_from_int (_IONBF));
 
+  sys_file_port_name_canonicalization = scm_make_fluid ();
+  scm_c_define ("%file-port-name-canonicalization",
+                sys_file_port_name_canonicalization);
+                                    
 #include "libguile/fports.x"
 }
 
