@@ -99,12 +99,24 @@ VM_DEFINE_FUNCTION (137, listp, "list?", 1)
   RETURN (scm_from_bool (scm_ilength (x) >= 0));
 }
 
+VM_DEFINE_FUNCTION (138, symbolp, "symbol?", 1)
+{
+  ARGS1 (x);
+  RETURN (scm_from_bool (scm_is_symbol (x)));
+}
+
+VM_DEFINE_FUNCTION (139, vectorp, "vector?", 1)
+{
+  ARGS1 (x);
+  RETURN (scm_from_bool (SCM_I_IS_VECTOR (x)));
+}
+
 
 /*
  * Basic data
  */
 
-VM_DEFINE_FUNCTION (138, cons, "cons", 2)
+VM_DEFINE_FUNCTION (140, cons, "cons", 2)
 {
   ARGS2 (x, y);
   CONS (x, x, y);
@@ -118,21 +130,21 @@ VM_DEFINE_FUNCTION (138, cons, "cons", 2)
       goto vm_error_not_a_pair;                 \
     }
   
-VM_DEFINE_FUNCTION (139, car, "car", 1)
+VM_DEFINE_FUNCTION (141, car, "car", 1)
 {
   ARGS1 (x);
   VM_VALIDATE_CONS (x, "car");
   RETURN (SCM_CAR (x));
 }
 
-VM_DEFINE_FUNCTION (140, cdr, "cdr", 1)
+VM_DEFINE_FUNCTION (142, cdr, "cdr", 1)
 {
   ARGS1 (x);
   VM_VALIDATE_CONS (x, "cdr");
   RETURN (SCM_CDR (x));
 }
 
-VM_DEFINE_INSTRUCTION (141, set_car, "set-car!", 0, 2, 0)
+VM_DEFINE_INSTRUCTION (143, set_car, "set-car!", 0, 2, 0)
 {
   SCM x, y;
   POP (y);
@@ -142,7 +154,7 @@ VM_DEFINE_INSTRUCTION (141, set_car, "set-car!", 0, 2, 0)
   NEXT;
 }
 
-VM_DEFINE_INSTRUCTION (142, set_cdr, "set-cdr!", 0, 2, 0)
+VM_DEFINE_INSTRUCTION (144, set_cdr, "set-cdr!", 0, 2, 0)
 {
   SCM x, y;
   POP (y);
@@ -167,27 +179,27 @@ VM_DEFINE_INSTRUCTION (142, set_cdr, "set-cdr!", 0, 2, 0)
   RETURN (srel (x, y));                                         \
 }
 
-VM_DEFINE_FUNCTION (143, ee, "ee?", 2)
+VM_DEFINE_FUNCTION (145, ee, "ee?", 2)
 {
   REL (==, scm_num_eq_p);
 }
 
-VM_DEFINE_FUNCTION (144, lt, "lt?", 2)
+VM_DEFINE_FUNCTION (146, lt, "lt?", 2)
 {
   REL (<, scm_less_p);
 }
 
-VM_DEFINE_FUNCTION (145, le, "le?", 2)
+VM_DEFINE_FUNCTION (147, le, "le?", 2)
 {
   REL (<=, scm_leq_p);
 }
 
-VM_DEFINE_FUNCTION (146, gt, "gt?", 2)
+VM_DEFINE_FUNCTION (148, gt, "gt?", 2)
 {
   REL (>, scm_gr_p);
 }
 
-VM_DEFINE_FUNCTION (147, ge, "ge?", 2)
+VM_DEFINE_FUNCTION (149, ge, "ge?", 2)
 {
   REL (>=, scm_geq_p);
 }
@@ -197,6 +209,10 @@ VM_DEFINE_FUNCTION (147, ge, "ge?", 2)
  * Numeric functions
  */
 
+/* The maximum/minimum tagged integers.  */
+#define INUM_MAX (INTPTR_MAX - 1)
+#define INUM_MIN (INTPTR_MIN + scm_tc2_int)
+
 #undef FUNC2
 #define FUNC2(CFUNC,SFUNC)				\
 {							\
@@ -211,78 +227,96 @@ VM_DEFINE_FUNCTION (147, ge, "ge?", 2)
   RETURN (SFUNC (x, y));				\
 }
 
-VM_DEFINE_FUNCTION (148, add, "add", 2)
+VM_DEFINE_FUNCTION (150, add, "add", 2)
 {
   FUNC2 (+, scm_sum);
 }
 
-VM_DEFINE_FUNCTION (149, add1, "add1", 1)
+VM_DEFINE_FUNCTION (151, add1, "add1", 1)
 {
   ARGS1 (x);
-  if (SCM_I_INUMP (x))
+
+  /* Check for overflow.  */
+  if (SCM_LIKELY ((scm_t_intptr) x < INUM_MAX))
     {
-      scm_t_int64 n = SCM_I_INUM (x) + 1;
-      if (SCM_FIXABLE (n))
-	RETURN (SCM_I_MAKINUM (n));
+      SCM result;
+
+      /* Add the integers without untagging.  */
+      result = SCM_PACK ((scm_t_intptr) x
+			 + (scm_t_intptr) SCM_I_MAKINUM (1)
+			 - scm_tc2_int);
+
+      if (SCM_LIKELY (SCM_I_INUMP (result)))
+	RETURN (result);
     }
+
   SYNC_REGISTER ();
   RETURN (scm_sum (x, SCM_I_MAKINUM (1)));
 }
 
-VM_DEFINE_FUNCTION (150, sub, "sub", 2)
+VM_DEFINE_FUNCTION (152, sub, "sub", 2)
 {
   FUNC2 (-, scm_difference);
 }
 
-VM_DEFINE_FUNCTION (151, sub1, "sub1", 1)
+VM_DEFINE_FUNCTION (153, sub1, "sub1", 1)
 {
   ARGS1 (x);
-  if (SCM_I_INUMP (x))
+
+  /* Check for underflow.  */
+  if (SCM_LIKELY ((scm_t_intptr) x > INUM_MIN))
     {
-      scm_t_int64 n = SCM_I_INUM (x) - 1;
-      if (SCM_FIXABLE (n))
-	RETURN (SCM_I_MAKINUM (n));
+      SCM result;
+
+      /* Substract the integers without untagging.  */
+      result = SCM_PACK ((scm_t_intptr) x
+			 - (scm_t_intptr) SCM_I_MAKINUM (1)
+			 + scm_tc2_int);
+
+      if (SCM_LIKELY (SCM_I_INUMP (result)))
+	RETURN (result);
     }
+
   SYNC_REGISTER ();
   RETURN (scm_difference (x, SCM_I_MAKINUM (1)));
 }
 
-VM_DEFINE_FUNCTION (152, mul, "mul", 2)
+VM_DEFINE_FUNCTION (154, mul, "mul", 2)
 {
   ARGS2 (x, y);
   SYNC_REGISTER ();
   RETURN (scm_product (x, y));
 }
 
-VM_DEFINE_FUNCTION (153, div, "div", 2)
+VM_DEFINE_FUNCTION (155, div, "div", 2)
 {
   ARGS2 (x, y);
   SYNC_REGISTER ();
   RETURN (scm_divide (x, y));
 }
 
-VM_DEFINE_FUNCTION (154, quo, "quo", 2)
+VM_DEFINE_FUNCTION (156, quo, "quo", 2)
 {
   ARGS2 (x, y);
   SYNC_REGISTER ();
   RETURN (scm_quotient (x, y));
 }
 
-VM_DEFINE_FUNCTION (155, rem, "rem", 2)
+VM_DEFINE_FUNCTION (157, rem, "rem", 2)
 {
   ARGS2 (x, y);
   SYNC_REGISTER ();
   RETURN (scm_remainder (x, y));
 }
 
-VM_DEFINE_FUNCTION (156, mod, "mod", 2)
+VM_DEFINE_FUNCTION (158, mod, "mod", 2)
 {
   ARGS2 (x, y);
   SYNC_REGISTER ();
   RETURN (scm_modulo (x, y));
 }
 
-VM_DEFINE_FUNCTION (157, ash, "ash", 2)
+VM_DEFINE_FUNCTION (159, ash, "ash", 2)
 {
   ARGS2 (x, y);
   if (SCM_I_INUMP (x) && SCM_I_INUMP (y))
@@ -311,7 +345,7 @@ VM_DEFINE_FUNCTION (157, ash, "ash", 2)
   RETURN (scm_ash (x, y));
 }
 
-VM_DEFINE_FUNCTION (158, logand, "logand", 2)
+VM_DEFINE_FUNCTION (160, logand, "logand", 2)
 {
   ARGS2 (x, y);
   if (SCM_I_INUMP (x) && SCM_I_INUMP (y))
@@ -320,7 +354,7 @@ VM_DEFINE_FUNCTION (158, logand, "logand", 2)
   RETURN (scm_logand (x, y));
 }
 
-VM_DEFINE_FUNCTION (159, logior, "logior", 2)
+VM_DEFINE_FUNCTION (161, logior, "logior", 2)
 {
   ARGS2 (x, y);
   if (SCM_I_INUMP (x) && SCM_I_INUMP (y))
@@ -329,7 +363,7 @@ VM_DEFINE_FUNCTION (159, logior, "logior", 2)
   RETURN (scm_logior (x, y));
 }
 
-VM_DEFINE_FUNCTION (160, logxor, "logxor", 2)
+VM_DEFINE_FUNCTION (162, logxor, "logxor", 2)
 {
   ARGS2 (x, y);
   if (SCM_I_INUMP (x) && SCM_I_INUMP (y))
@@ -343,7 +377,7 @@ VM_DEFINE_FUNCTION (160, logxor, "logxor", 2)
  * Vectors and arrays
  */
 
-VM_DEFINE_FUNCTION (161, vector_ref, "vector-ref", 2)
+VM_DEFINE_FUNCTION (163, vector_ref, "vector-ref", 2)
 {
   long i = 0;
   ARGS2 (vect, idx);
@@ -359,7 +393,7 @@ VM_DEFINE_FUNCTION (161, vector_ref, "vector-ref", 2)
     }
 }
 
-VM_DEFINE_INSTRUCTION (162, vector_set, "vector-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (164, vector_set, "vector-set", 0, 3, 0)
 {
   long i = 0;
   SCM vect, idx, val;
@@ -377,7 +411,7 @@ VM_DEFINE_INSTRUCTION (162, vector_set, "vector-set", 0, 3, 0)
   NEXT;
 }
 
-VM_DEFINE_INSTRUCTION (163, make_array, "make-array", 3, -1, 1)
+VM_DEFINE_INSTRUCTION (165, make_array, "make-array", 3, -1, 1)
 {
   scm_t_uint32 len;
   SCM shape, ret;
@@ -406,20 +440,20 @@ VM_DEFINE_INSTRUCTION (163, make_array, "make-array", 3, -1, 1)
       goto vm_error_not_a_struct;		\
     }
 
-VM_DEFINE_FUNCTION (164, struct_p, "struct?", 1)
+VM_DEFINE_FUNCTION (166, struct_p, "struct?", 1)
 {
   ARGS1 (obj);
   RETURN (scm_from_bool (SCM_STRUCTP (obj)));
 }
 
-VM_DEFINE_FUNCTION (165, struct_vtable, "struct-vtable", 1)
+VM_DEFINE_FUNCTION (167, struct_vtable, "struct-vtable", 1)
 {
   ARGS1 (obj);
   VM_VALIDATE_STRUCT (obj, "struct_vtable");
   RETURN (SCM_STRUCT_VTABLE (obj));
 }
 
-VM_DEFINE_INSTRUCTION (166, make_struct, "make-struct", 2, -1, 1)
+VM_DEFINE_INSTRUCTION (168, make_struct, "make-struct", 2, -1, 1)
 {
   unsigned h = FETCH ();
   unsigned l = FETCH ();
@@ -452,7 +486,7 @@ VM_DEFINE_INSTRUCTION (166, make_struct, "make-struct", 2, -1, 1)
   NEXT;
 }
 
-VM_DEFINE_FUNCTION (167, struct_ref, "struct-ref", 2)
+VM_DEFINE_FUNCTION (169, struct_ref, "struct-ref", 2)
 {
   ARGS2 (obj, pos);
 
@@ -479,7 +513,7 @@ VM_DEFINE_FUNCTION (167, struct_ref, "struct-ref", 2)
   RETURN (scm_struct_ref (obj, pos));
 }
 
-VM_DEFINE_FUNCTION (168, struct_set, "struct-set", 3)
+VM_DEFINE_FUNCTION (170, struct_set, "struct-set", 3)
 {
   ARGS3 (obj, pos, val);
 
@@ -512,7 +546,7 @@ VM_DEFINE_FUNCTION (168, struct_set, "struct-set", 3)
 /*
  * GOOPS support
  */
-VM_DEFINE_FUNCTION (169, class_of, "class-of", 1)
+VM_DEFINE_FUNCTION (171, class_of, "class-of", 1)
 {
   ARGS1 (obj);
   if (SCM_INSTANCEP (obj))
@@ -521,7 +555,7 @@ VM_DEFINE_FUNCTION (169, class_of, "class-of", 1)
   RETURN (scm_class_of (obj));
 }
 
-VM_DEFINE_FUNCTION (170, slot_ref, "slot-ref", 2)
+VM_DEFINE_FUNCTION (172, slot_ref, "slot-ref", 2)
 {
   size_t slot;
   ARGS2 (instance, idx);
@@ -529,7 +563,7 @@ VM_DEFINE_FUNCTION (170, slot_ref, "slot-ref", 2)
   RETURN (SCM_PACK (SCM_STRUCT_DATA (instance) [slot]));
 }
 
-VM_DEFINE_INSTRUCTION (171, slot_set, "slot-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (173, slot_set, "slot-set", 0, 3, 0)
 {
   SCM instance, idx, val;
   size_t slot;
@@ -574,21 +608,21 @@ VM_DEFINE_INSTRUCTION (171, slot_set, "slot-set", 0, 3, 0)
 #define ALIGNED_P(ptr, type)			\
   ((scm_t_uintptr) (ptr) % alignof (type) == 0)
 
-VM_DEFINE_FUNCTION (172, bv_u16_ref, "bv-u16-ref", 3)
+VM_DEFINE_FUNCTION (174, bv_u16_ref, "bv-u16-ref", 3)
 BV_REF_WITH_ENDIANNESS (u16, u16)
-VM_DEFINE_FUNCTION (173, bv_s16_ref, "bv-s16-ref", 3)
+VM_DEFINE_FUNCTION (175, bv_s16_ref, "bv-s16-ref", 3)
 BV_REF_WITH_ENDIANNESS (s16, s16)
-VM_DEFINE_FUNCTION (174, bv_u32_ref, "bv-u32-ref", 3)
+VM_DEFINE_FUNCTION (176, bv_u32_ref, "bv-u32-ref", 3)
 BV_REF_WITH_ENDIANNESS (u32, u32)
-VM_DEFINE_FUNCTION (175, bv_s32_ref, "bv-s32-ref", 3)
+VM_DEFINE_FUNCTION (177, bv_s32_ref, "bv-s32-ref", 3)
 BV_REF_WITH_ENDIANNESS (s32, s32)
-VM_DEFINE_FUNCTION (176, bv_u64_ref, "bv-u64-ref", 3)
+VM_DEFINE_FUNCTION (178, bv_u64_ref, "bv-u64-ref", 3)
 BV_REF_WITH_ENDIANNESS (u64, u64)
-VM_DEFINE_FUNCTION (177, bv_s64_ref, "bv-s64-ref", 3)
+VM_DEFINE_FUNCTION (179, bv_s64_ref, "bv-s64-ref", 3)
 BV_REF_WITH_ENDIANNESS (s64, s64)
-VM_DEFINE_FUNCTION (178, bv_f32_ref, "bv-f32-ref", 3)
+VM_DEFINE_FUNCTION (180, bv_f32_ref, "bv-f32-ref", 3)
 BV_REF_WITH_ENDIANNESS (f32, ieee_single)
-VM_DEFINE_FUNCTION (179, bv_f64_ref, "bv-f64-ref", 3)
+VM_DEFINE_FUNCTION (181, bv_f64_ref, "bv-f64-ref", 3)
 BV_REF_WITH_ENDIANNESS (f64, ieee_double)
 
 #undef BV_REF_WITH_ENDIANNESS
@@ -666,33 +700,33 @@ BV_REF_WITH_ENDIANNESS (f64, ieee_double)
     RETURN (scm_bytevector_ ## fn_stem ## _native_ref (bv, idx));	\
 }
 
-VM_DEFINE_FUNCTION (180, bv_u8_ref, "bv-u8-ref", 2)
+VM_DEFINE_FUNCTION (182, bv_u8_ref, "bv-u8-ref", 2)
 BV_FIXABLE_INT_REF (u8, u8, uint8, 1)
-VM_DEFINE_FUNCTION (181, bv_s8_ref, "bv-s8-ref", 2)
+VM_DEFINE_FUNCTION (183, bv_s8_ref, "bv-s8-ref", 2)
 BV_FIXABLE_INT_REF (s8, s8, int8, 1)
-VM_DEFINE_FUNCTION (182, bv_u16_native_ref, "bv-u16-native-ref", 2)
+VM_DEFINE_FUNCTION (184, bv_u16_native_ref, "bv-u16-native-ref", 2)
 BV_FIXABLE_INT_REF (u16, u16_native, uint16, 2)
-VM_DEFINE_FUNCTION (183, bv_s16_native_ref, "bv-s16-native-ref", 2)
+VM_DEFINE_FUNCTION (185, bv_s16_native_ref, "bv-s16-native-ref", 2)
 BV_FIXABLE_INT_REF (s16, s16_native, int16, 2)
-VM_DEFINE_FUNCTION (184, bv_u32_native_ref, "bv-u32-native-ref", 2)
+VM_DEFINE_FUNCTION (186, bv_u32_native_ref, "bv-u32-native-ref", 2)
 #if SIZEOF_VOID_P > 4
 BV_FIXABLE_INT_REF (u32, u32_native, uint32, 4)
 #else
 BV_INT_REF (u32, uint32, 4)
 #endif
-VM_DEFINE_FUNCTION (185, bv_s32_native_ref, "bv-s32-native-ref", 2)
+VM_DEFINE_FUNCTION (187, bv_s32_native_ref, "bv-s32-native-ref", 2)
 #if SIZEOF_VOID_P > 4
 BV_FIXABLE_INT_REF (s32, s32_native, int32, 4)
 #else
 BV_INT_REF (s32, int32, 4)
 #endif
-VM_DEFINE_FUNCTION (186, bv_u64_native_ref, "bv-u64-native-ref", 2)
+VM_DEFINE_FUNCTION (188, bv_u64_native_ref, "bv-u64-native-ref", 2)
 BV_INT_REF (u64, uint64, 8)
-VM_DEFINE_FUNCTION (187, bv_s64_native_ref, "bv-s64-native-ref", 2)
+VM_DEFINE_FUNCTION (189, bv_s64_native_ref, "bv-s64-native-ref", 2)
 BV_INT_REF (s64, int64, 8)
-VM_DEFINE_FUNCTION (188, bv_f32_native_ref, "bv-f32-native-ref", 2)
+VM_DEFINE_FUNCTION (190, bv_f32_native_ref, "bv-f32-native-ref", 2)
 BV_FLOAT_REF (f32, ieee_single, float, 4)
-VM_DEFINE_FUNCTION (189, bv_f64_native_ref, "bv-f64-native-ref", 2)
+VM_DEFINE_FUNCTION (191, bv_f64_native_ref, "bv-f64-native-ref", 2)
 BV_FLOAT_REF (f64, ieee_double, double, 8)
 
 #undef BV_FIXABLE_INT_REF
@@ -714,21 +748,21 @@ BV_FLOAT_REF (f64, ieee_double, double, 8)
   }                                                                     \
 }
 
-VM_DEFINE_INSTRUCTION (190, bv_u16_set, "bv-u16-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (192, bv_u16_set, "bv-u16-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (u16, u16)
-VM_DEFINE_INSTRUCTION (191, bv_s16_set, "bv-s16-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (193, bv_s16_set, "bv-s16-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (s16, s16)
-VM_DEFINE_INSTRUCTION (192, bv_u32_set, "bv-u32-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (194, bv_u32_set, "bv-u32-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (u32, u32)
-VM_DEFINE_INSTRUCTION (193, bv_s32_set, "bv-s32-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (195, bv_s32_set, "bv-s32-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (s32, s32)
-VM_DEFINE_INSTRUCTION (194, bv_u64_set, "bv-u64-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (196, bv_u64_set, "bv-u64-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (u64, u64)
-VM_DEFINE_INSTRUCTION (195, bv_s64_set, "bv-s64-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (197, bv_s64_set, "bv-s64-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (s64, s64)
-VM_DEFINE_INSTRUCTION (196, bv_f32_set, "bv-f32-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (198, bv_f32_set, "bv-f32-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (f32, ieee_single)
-VM_DEFINE_INSTRUCTION (197, bv_f64_set, "bv-f64-set", 0, 4, 0)
+VM_DEFINE_INSTRUCTION (199, bv_f64_set, "bv-f64-set", 0, 4, 0)
 BV_SET_WITH_ENDIANNESS (f64, ieee_double)
 
 #undef BV_SET_WITH_ENDIANNESS
@@ -799,33 +833,33 @@ BV_SET_WITH_ENDIANNESS (f64, ieee_double)
   NEXT;								\
 }
 
-VM_DEFINE_INSTRUCTION (198, bv_u8_set, "bv-u8-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (200, bv_u8_set, "bv-u8-set", 0, 3, 0)
 BV_FIXABLE_INT_SET (u8, u8, uint8, 0, SCM_T_UINT8_MAX, 1)
-VM_DEFINE_INSTRUCTION (199, bv_s8_set, "bv-s8-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (201, bv_s8_set, "bv-s8-set", 0, 3, 0)
 BV_FIXABLE_INT_SET (s8, s8, int8, SCM_T_INT8_MIN, SCM_T_INT8_MAX, 1)
-VM_DEFINE_INSTRUCTION (200, bv_u16_native_set, "bv-u16-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (202, bv_u16_native_set, "bv-u16-native-set", 0, 3, 0)
 BV_FIXABLE_INT_SET (u16, u16_native, uint16, 0, SCM_T_UINT16_MAX, 2)
-VM_DEFINE_INSTRUCTION (201, bv_s16_native_set, "bv-s16-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (203, bv_s16_native_set, "bv-s16-native-set", 0, 3, 0)
 BV_FIXABLE_INT_SET (s16, s16_native, int16, SCM_T_INT16_MIN, SCM_T_INT16_MAX, 2)
-VM_DEFINE_INSTRUCTION (202, bv_u32_native_set, "bv-u32-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (204, bv_u32_native_set, "bv-u32-native-set", 0, 3, 0)
 #if SIZEOF_VOID_P > 4
 BV_FIXABLE_INT_SET (u32, u32_native, uint32, 0, SCM_T_UINT32_MAX, 4)
 #else
 BV_INT_SET (u32, uint32, 4)
 #endif
-VM_DEFINE_INSTRUCTION (203, bv_s32_native_set, "bv-s32-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (205, bv_s32_native_set, "bv-s32-native-set", 0, 3, 0)
 #if SIZEOF_VOID_P > 4
 BV_FIXABLE_INT_SET (s32, s32_native, int32, SCM_T_INT32_MIN, SCM_T_INT32_MAX, 4)
 #else
 BV_INT_SET (s32, int32, 4)
 #endif
-VM_DEFINE_INSTRUCTION (204, bv_u64_native_set, "bv-u64-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (206, bv_u64_native_set, "bv-u64-native-set", 0, 3, 0)
 BV_INT_SET (u64, uint64, 8)
-VM_DEFINE_INSTRUCTION (205, bv_s64_native_set, "bv-s64-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (207, bv_s64_native_set, "bv-s64-native-set", 0, 3, 0)
 BV_INT_SET (s64, int64, 8)
-VM_DEFINE_INSTRUCTION (206, bv_f32_native_set, "bv-f32-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (208, bv_f32_native_set, "bv-f32-native-set", 0, 3, 0)
 BV_FLOAT_SET (f32, ieee_single, float, 4)
-VM_DEFINE_INSTRUCTION (207, bv_f64_native_set, "bv-f64-native-set", 0, 3, 0)
+VM_DEFINE_INSTRUCTION (209, bv_f64_native_set, "bv-f64-native-set", 0, 3, 0)
 BV_FLOAT_SET (f64, ieee_double, double, 8)
 
 #undef BV_FIXABLE_INT_SET
