@@ -24,6 +24,7 @@
 #include <alloca.h>
 #include <alignof.h>
 #include <string.h>
+#include <stdint.h>
 
 #include "libguile/bdw-gc.h"
 #include <gc/gc_mark.h>
@@ -36,12 +37,15 @@
 #include "programs.h"
 #include "vm.h"
 
-/* I sometimes use this for debugging. */
-#define vm_puts(OBJ)				\
-{						\
-  scm_display (OBJ, scm_current_error_port ()); \
-  scm_newline (scm_current_error_port ());      \
-}
+static int vm_default_engine = SCM_VM_REGULAR_ENGINE;
+
+/* Unfortunately we can't snarf these: snarfed things are only loaded up from
+   (system vm vm), which might not be loaded before an error happens. */
+static SCM sym_vm_run;
+static SCM sym_vm_error;
+static SCM sym_keyword_argument_error;
+static SCM sym_regular;
+static SCM sym_debug;
 
 /* The VM has a number of internal assertions that shouldn't normally be
    necessary, but might be if you think you found a bug in the VM. */
@@ -191,8 +195,9 @@ vm_dispatch_hook (SCM vm, int hook_num)
   struct scm_vm *vp;
   SCM hook;
   struct scm_frame c_frame;
-  scm_t_aligned_cell frame;
+  scm_t_cell *frame;
   SCM args[1];
+  int saved_trace_level;
 
   vp = SCM_VM_DATA (vm);
   hook = vp->hooks[hook_num];
@@ -201,7 +206,8 @@ vm_dispatch_hook (SCM vm, int hook_num)
       || scm_is_null (SCM_HOOK_PROCEDURES (hook)))
     return;
 
-  vp->trace_level--;
+  saved_trace_level = vp->trace_level;
+  vp->trace_level = 0;
 
   /* Allocate a frame object on the stack.  This is more efficient than calling
      `scm_c_make_frame ()' to allocate on the heap, but it forces hooks to not
@@ -216,13 +222,18 @@ vm_dispatch_hook (SCM vm, int hook_num)
   c_frame.sp = vp->sp;
   c_frame.ip = vp->ip;
   c_frame.offset = 0;
-  frame.cell.word_0 = SCM_PACK (scm_tc7_frame);
-  frame.cell.word_1 = PTR2SCM (&c_frame);
-  args[0] = PTR2SCM (&frame);
+
+  /* Arrange for FRAME to be 8-byte aligned, like any other cell.  */
+  frame = alloca (sizeof (*frame) + 8);
+  frame = (scm_t_cell *) ROUND_UP ((scm_t_uintptr) frame, 8UL);
+
+  frame->word_0 = SCM_PACK (scm_tc7_frame);
+  frame->word_1 = PTR2SCM (&c_frame);
+  args[0] = PTR2SCM (frame);
 
   scm_c_run_hookn (hook, args, 1);
 
-  vp->trace_level++;
+  vp->trace_level = saved_trace_level;
 }
 
 static void vm_abort (SCM vm, size_t n, scm_t_int64 cookie) SCM_NORETURN;
@@ -333,10 +344,6 @@ vm_reinstate_partial_continuation (SCM vm, SCM cont, SCM intwinds,
  * VM Internal functions
  */
 
-/* Unfortunately we can't snarf these: snarfed things are only loaded up from
-   (system vm vm), which might not be loaded before an error happens. */
-static SCM sym_vm_run, sym_vm_error, sym_keyword_argument_error, sym_debug;
-
 void
 scm_i_vm_print (SCM x, SCM port, scm_print_state *pstate)
 {
@@ -510,8 +517,7 @@ make_vm (void)
   vp->ip    	  = NULL;
   vp->sp    	  = vp->stack_base - 1;
   vp->fp    	  = NULL;
-  vp->engine      = SCM_VM_DEBUG_ENGINE;
-  vp->options     = SCM_EOL;
+  vp->engine      = vm_default_engine;
   vp->trace_level = 0;
   for (i = 0; i < SCM_VM_NUM_HOOKS; i++)
     vp->hooks[i] = SCM_BOOL_F;
@@ -555,54 +561,20 @@ SCM
 scm_c_vm_run (SCM vm, SCM program, SCM *argv, int nargs)
 {
   struct scm_vm *vp = SCM_VM_DATA (vm);
+  SCM_CHECK_STACK;
   return vm_engines[vp->engine](vm, program, argv, nargs);
 }
 
-SCM_DEFINE (scm_vm_apply, "vm-apply", 3, 0, 0,
-            (SCM vm, SCM program, SCM args),
-            "")
-#define FUNC_NAME s_scm_vm_apply
-{
-  SCM *argv;
-  int i, nargs;
-  
-  SCM_VALIDATE_VM (1, vm);
-  SCM_VALIDATE_PROC (2, program);
-
-  nargs = scm_ilength (args);
-  if (SCM_UNLIKELY (nargs < 0))
-    scm_wrong_type_arg_msg (FUNC_NAME, 3, args, "list");
-  
-  argv = alloca(nargs * sizeof(SCM));
-  for (i = 0; i < nargs; i++)
-    {
-      argv[i] = SCM_CAR (args);
-      args = SCM_CDR (args);
-    }
-
-  return scm_c_vm_run (vm, program, argv, nargs);
-}
-#undef FUNC_NAME
-
 /* Scheme interface */
 
-SCM_DEFINE (scm_vm_version, "vm-version", 0, 0, 0,
-	    (void),
-	    "")
-#define FUNC_NAME s_scm_vm_version
-{
-  return scm_from_locale_string (PACKAGE_VERSION);
-}
-#undef FUNC_NAME
-
 SCM_DEFINE (scm_the_vm, "the-vm", 0, 0, 0,
 	    (void),
-	    "")
+	    "Return the current thread's VM.")
 #define FUNC_NAME s_scm_the_vm
 {
   scm_i_thread *t = SCM_I_CURRENT_THREAD;
 
-  if (SCM_UNLIKELY (scm_is_false ((t->vm))))
+  if (SCM_UNLIKELY (scm_is_false (t->vm)))
     t->vm = make_vm ();
 
   return t->vm;
@@ -668,21 +640,30 @@ SCM_DEFINE (scm_vm_fp, "vm:fp", 1, 0, 0,
   return vp->hooks[n];					\
 }
 
-SCM_DEFINE (scm_vm_boot_hook, "vm-boot-hook", 1, 0, 0,
+SCM_DEFINE (scm_vm_apply_hook, "vm-apply-hook", 1, 0, 0,
 	    (SCM vm),
 	    "")
-#define FUNC_NAME s_scm_vm_boot_hook
+#define FUNC_NAME s_scm_vm_apply_hook
 {
-  VM_DEFINE_HOOK (SCM_VM_BOOT_HOOK);
+  VM_DEFINE_HOOK (SCM_VM_APPLY_HOOK);
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_vm_halt_hook, "vm-halt-hook", 1, 0, 0,
+SCM_DEFINE (scm_vm_push_continuation_hook, "vm-push-continuation-hook", 1, 0, 0,
 	    (SCM vm),
 	    "")
-#define FUNC_NAME s_scm_vm_halt_hook
+#define FUNC_NAME s_scm_vm_push_continuation_hook
 {
-  VM_DEFINE_HOOK (SCM_VM_HALT_HOOK);
+  VM_DEFINE_HOOK (SCM_VM_PUSH_CONTINUATION_HOOK);
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_vm_pop_continuation_hook, "vm-pop-continuation-hook", 1, 0, 0,
+	    (SCM vm),
+	    "")
+#define FUNC_NAME s_scm_vm_pop_continuation_hook
+{
+  VM_DEFINE_HOOK (SCM_VM_POP_CONTINUATION_HOOK);
 }
 #undef FUNC_NAME
 
@@ -695,70 +676,21 @@ SCM_DEFINE (scm_vm_next_hook, "vm-next-hook", 1, 0, 0,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_vm_break_hook, "vm-break-hook", 1, 0, 0,
+SCM_DEFINE (scm_vm_abort_continuation_hook, "vm-abort-continuation-hook", 1, 0, 0,
 	    (SCM vm),
 	    "")
-#define FUNC_NAME s_scm_vm_break_hook
+#define FUNC_NAME s_scm_vm_abort_continuation_hook
 {
-  VM_DEFINE_HOOK (SCM_VM_BREAK_HOOK);
+  VM_DEFINE_HOOK (SCM_VM_ABORT_CONTINUATION_HOOK);
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_vm_enter_hook, "vm-enter-hook", 1, 0, 0,
+SCM_DEFINE (scm_vm_restore_continuation_hook, "vm-restore-continuation-hook", 1, 0, 0,
 	    (SCM vm),
 	    "")
-#define FUNC_NAME s_scm_vm_enter_hook
+#define FUNC_NAME s_scm_vm_restore_continuation_hook
 {
-  VM_DEFINE_HOOK (SCM_VM_ENTER_HOOK);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_vm_apply_hook, "vm-apply-hook", 1, 0, 0,
-	    (SCM vm),
-	    "")
-#define FUNC_NAME s_scm_vm_apply_hook
-{
-  VM_DEFINE_HOOK (SCM_VM_APPLY_HOOK);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_vm_exit_hook, "vm-exit-hook", 1, 0, 0,
-	    (SCM vm),
-	    "")
-#define FUNC_NAME s_scm_vm_exit_hook
-{
-  VM_DEFINE_HOOK (SCM_VM_EXIT_HOOK);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_vm_return_hook, "vm-return-hook", 1, 0, 0,
-	    (SCM vm),
-	    "")
-#define FUNC_NAME s_scm_vm_return_hook
-{
-  VM_DEFINE_HOOK (SCM_VM_RETURN_HOOK);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_vm_option, "vm-option", 2, 0, 0,
-	    (SCM vm, SCM key),
-	    "")
-#define FUNC_NAME s_scm_vm_option
-{
-  SCM_VALIDATE_VM (1, vm);
-  return scm_assq_ref (SCM_VM_DATA (vm)->options, key);
-}
-#undef FUNC_NAME
-
-SCM_DEFINE (scm_set_vm_option_x, "set-vm-option!", 3, 0, 0,
-	    (SCM vm, SCM key, SCM val),
-	    "")
-#define FUNC_NAME s_scm_set_vm_option_x
-{
-  SCM_VALIDATE_VM (1, vm);
-  SCM_VM_DATA (vm)->options
-    = scm_assq_set_x (SCM_VM_DATA (vm)->options, key, val);
-  return SCM_UNSPECIFIED;
+  VM_DEFINE_HOOK (SCM_VM_RESTORE_CONTINUATION_HOOK);
 }
 #undef FUNC_NAME
 
@@ -783,6 +715,154 @@ SCM_DEFINE (scm_set_vm_trace_level_x, "set-vm-trace-level!", 2, 0, 0,
 }
 #undef FUNC_NAME
 
+
+/*
+ * VM engines
+ */
+
+static int
+symbol_to_vm_engine (SCM engine, const char *FUNC_NAME)
+{
+  if (scm_is_eq (engine, sym_regular))
+    return SCM_VM_REGULAR_ENGINE;
+  else if (scm_is_eq (engine, sym_debug))
+    return SCM_VM_DEBUG_ENGINE;
+  else
+    SCM_MISC_ERROR ("Unknown VM engine: ~a", scm_list_1 (engine));
+}
+  
+static SCM
+vm_engine_to_symbol (int engine, const char *FUNC_NAME)
+{
+  switch (engine)
+    {
+    case SCM_VM_REGULAR_ENGINE:
+      return sym_regular;
+    case SCM_VM_DEBUG_ENGINE:
+      return sym_debug;
+    default:
+      /* ? */
+      SCM_MISC_ERROR ("Unknown VM engine: ~a",
+                      scm_list_1 (scm_from_int (engine)));
+    }
+}
+  
+SCM_DEFINE (scm_vm_engine, "vm-engine", 1, 0, 0,
+	    (SCM vm),
+	    "")
+#define FUNC_NAME s_scm_vm_engine
+{
+  SCM_VALIDATE_VM (1, vm);
+  return vm_engine_to_symbol (SCM_VM_DATA (vm)->engine, FUNC_NAME);
+}
+#undef FUNC_NAME
+
+void
+scm_c_set_vm_engine_x (SCM vm, int engine)
+#define FUNC_NAME "set-vm-engine!"
+{
+  SCM_VALIDATE_VM (1, vm);
+
+  if (engine < 0 || engine >= SCM_VM_NUM_ENGINES)
+    SCM_MISC_ERROR ("Unknown VM engine: ~a",
+                    scm_list_1 (scm_from_int (engine)));
+    
+  SCM_VM_DATA (vm)->engine = engine;
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_set_vm_engine_x, "set-vm-engine!", 2, 0, 0,
+	    (SCM vm, SCM engine),
+	    "")
+#define FUNC_NAME s_scm_set_vm_engine_x
+{
+  scm_c_set_vm_engine_x (vm, symbol_to_vm_engine (engine, FUNC_NAME));
+  return SCM_UNSPECIFIED;
+}
+#undef FUNC_NAME
+
+void
+scm_c_set_default_vm_engine_x (int engine)
+#define FUNC_NAME "set-default-vm-engine!"
+{
+  if (engine < 0 || engine >= SCM_VM_NUM_ENGINES)
+    SCM_MISC_ERROR ("Unknown VM engine: ~a",
+                    scm_list_1 (scm_from_int (engine)));
+    
+  vm_default_engine = engine;
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_set_default_vm_engine_x, "set-default-vm-engine!", 1, 0, 0,
+	    (SCM engine),
+	    "")
+#define FUNC_NAME s_scm_set_default_vm_engine_x
+{
+  scm_c_set_default_vm_engine_x (symbol_to_vm_engine (engine, FUNC_NAME));
+  return SCM_UNSPECIFIED;
+}
+#undef FUNC_NAME
+
+static void reinstate_vm (SCM vm)
+{
+  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  t->vm = vm;
+}
+
+SCM_DEFINE (scm_call_with_vm, "call-with-vm", 2, 0, 1,
+	    (SCM vm, SCM proc, SCM args),
+	    "Apply @var{proc} to @var{args} in a dynamic extent in which\n"
+            "@var{vm} is the current VM.\n\n"
+            "As an implementation restriction, if @var{vm} is not the same\n"
+            "as the current thread's VM, continuations captured within the\n"
+            "call to @var{proc} may not be reinstated once control leaves\n"
+            "@var{proc}.")
+#define FUNC_NAME s_scm_call_with_vm
+{
+  SCM prev_vm, ret;
+  SCM *argv;
+  int i, nargs;
+  scm_t_wind_flags flags;
+  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+
+  SCM_VALIDATE_VM (1, vm);
+  SCM_VALIDATE_PROC (2, proc);
+
+  nargs = scm_ilength (args);
+  if (SCM_UNLIKELY (nargs < 0))
+    scm_wrong_type_arg_msg (FUNC_NAME, 3, args, "list");
+  
+  argv = alloca (nargs * sizeof(SCM));
+  for (i = 0; i < nargs; i++)
+    {
+      argv[i] = SCM_CAR (args);
+      args = SCM_CDR (args);
+    }
+
+  prev_vm = t->vm;
+
+  /* Reentry can happen via invokation of a saved continuation, but
+     continuations only save the state of the VM that they are in at
+     capture-time, which might be different from this one.  So, in the
+     case that the VMs are different, set up a non-rewindable frame to
+     prevent reinstating an incomplete continuation.  */
+  flags = scm_is_eq (prev_vm, vm) ? 0 : SCM_F_WIND_EXPLICITLY;
+  if (flags)
+    {
+      scm_dynwind_begin (0);
+      scm_dynwind_unwind_handler_with_scm (reinstate_vm, prev_vm, flags);
+      t->vm = vm;
+    }
+
+  ret = scm_c_vm_run (vm, proc, argv, nargs);
+
+  if (flags)
+    scm_dynwind_end ();
+  
+  return ret;
+}
+#undef FUNC_NAME
+
 
 /*
  * Initialize
@@ -806,6 +886,7 @@ scm_bootstrap_vm (void)
   sym_vm_run = scm_from_locale_symbol ("vm-run");
   sym_vm_error = scm_from_locale_symbol ("vm-error");
   sym_keyword_argument_error = scm_from_locale_symbol ("keyword-argument-error");
+  sym_regular = scm_from_locale_symbol ("regular");
   sym_debug = scm_from_locale_symbol ("debug");
 
 #ifdef VM_ENABLE_PRECISE_STACK_GC_SCAN
