@@ -64,7 +64,8 @@
             the-last-stack
             save-stack
             named-module-use!
-            top-repl))
+            top-repl
+            turn-on-debugging))
 
 
 ;;;; Deprecated definitions.
@@ -663,3 +664,21 @@ it.")
   (issue-deprecation-warning
    "`top-repl' has moved to the `(ice-9 top-repl)' module.")
   ((module-ref (resolve-module '(ice-9 top-repl)) 'top-repl)))
+
+(set! debug-enable
+      (let ((debug-enable debug-enable))
+        (lambda opts
+          (if (memq 'debug opts)
+              (begin
+                (issue-deprecation-warning
+                 "`(debug-enable 'debug)' is obsolete and has no effect."
+                 "Remove it from your code.")
+                (apply debug-enable (delq 'debug opts)))
+              (apply debug-enable opts)))))
+
+(define (turn-on-debugging)
+  (issue-deprecation-warning
+   "`(turn-on-debugging)' is obsolete and usually has no effect."
+   "Debugging capabilities are present by default.")
+  (debug-enable 'backtrace)
+  (read-enable 'positions))
