@@ -404,18 +404,10 @@ It is an image under the mapping EXTRACT."
 		     identity))
 
 (define (root-modules)
-  (cons the-root-module
-	(submodules (nested-ref the-root-module '(app modules)))))
+  (submodules (resolve-module '() #f)))
 
-(define (submodules m)
-  (hash-fold (lambda (name var data)
-	       (let ((obj (and (variable-bound? var) (variable-ref var))))
-		 (if (and (module? obj)
-			  (eq? (module-kind obj) 'directory))
-		     (cons obj data)
-		     data)))
-	     '()
-	     (module-obarray m)))
+(define (submodules mod)
+  (hash-map->list (lambda (k v) v) (module-submodules mod)))
 
 (define apropos-fold-exported
   (make-fold-modules root-modules submodules module-public-interface))
@@ -484,7 +476,7 @@ It is an image under the mapping EXTRACT."
 		  (display rest-arg)
 		  (display "'"))))))
    (else
-    (let ((arity (procedure-property obj 'arity)))
+    (let ((arity (procedure-minimum-arity obj)))
       (display (car arity))
       (cond ((caddr arity)
 	     (display " or more"))
