@@ -24,10 +24,13 @@
   #:use-module (system base language)
   #:use-module (system vm program)
   #:use-module (ice-9 control)
+  #:use-module (ice-9 history)
   #:export (<repl> make-repl repl-language repl-options
-            repl-tm-stats repl-gc-stats
-            repl-welcome repl-prompt repl-read repl-compile repl-eval
+            repl-tm-stats repl-gc-stats repl-inport repl-outport repl-debug
+            repl-welcome repl-prompt
+            repl-read repl-compile repl-prepare-eval-thunk repl-eval
             repl-parse repl-print repl-option-ref repl-option-set!
+            repl-default-option-set! repl-default-prompt-set!
             puts ->string user-error
             *warranty* *copying* *version*))
 
@@ -98,18 +101,36 @@ See <http://www.gnu.org/licenses/lgpl.html>, for more details.")
 ;;; Repl type
 ;;;
 
-(define-record/keywords <repl> language options tm-stats gc-stats)
+(define-record/keywords <repl>
+  language options tm-stats gc-stats inport outport debug)
 
 (define repl-default-options
-  '((trace . #f)
-    (interp . #f)))
+  (copy-tree
+   `((compile-options (#:warnings (unbound-variable arity-mismatch)) #f)
+     (trace #f #f)
+     (interp #f #f)
+     (prompt #f ,(lambda (prompt)
+                   (cond
+                    ((not prompt) #f)
+                    ((string? prompt) (lambda (repl) prompt))
+                    ((thunk? prompt) (lambda (repl) (prompt)))
+                    ((procedure? prompt) prompt)
+                    (else (error "Invalid prompt" prompt)))))
+     (value-history
+      ,(value-history-enabled?)
+      ,(lambda (x)
+         (if x (enable-value-history!) (disable-value-history!))
+         (->bool x))))))
 
 (define %make-repl make-repl)
-(define (make-repl lang)
+(define* (make-repl lang #:optional debug)
   (%make-repl #:language (lookup-language lang)
-              #:options repl-default-options
+              #:options (copy-tree repl-default-options)
               #:tm-stats (times)
-              #:gc-stats (gc-stats)))
+              #:gc-stats (gc-stats)
+              #:inport (current-input-port)
+              #:outport (current-output-port)
+              #:debug debug))
 
 (define (repl-welcome repl)
   (display *version*)
@@ -118,49 +139,79 @@ See <http://www.gnu.org/licenses/lgpl.html>, for more details.")
   (display "Enter `,help' for help.\n"))
 
 (define (repl-prompt repl)
-  (format #f "~A@~A> " (language-name (repl-language repl))
-          (module-name (current-module))))
+  (cond
+   ((repl-option-ref repl 'prompt)
+    => (lambda (prompt) (prompt repl)))
+   (else
+    (format #f "~A@~A~A> " (language-name (repl-language repl))
+            (module-name (current-module))
+            (let ((level (length (cond
+                                  ((fluid-ref *repl-stack*) => cdr)
+                                  (else '())))))
+              (if (zero? level) "" (format #f " [~a]" level)))))))
 
 (define (repl-read repl)
-  ((language-reader (repl-language repl)) (current-input-port)
+  ((language-reader (repl-language repl)) (repl-inport repl)
                                           (current-module)))
 
-(define (repl-compile repl form . opts)
-  (let ((to (lookup-language (cond ((memq #:e opts) 'scheme)
-                                   ((memq #:t opts) 'ghil)
-                                   ((memq #:c opts) 'glil)
-                                   (else 'objcode))))
-        (from (repl-language repl)))
-    (compile form #:from from #:to to #:opts opts #:env (current-module))))
+(define (repl-compile-options repl)
+  (repl-option-ref repl 'compile-options))
+
+(define (repl-compile repl form)
+  (let ((from (repl-language repl))
+        (opts (repl-compile-options repl)))
+    (compile form #:from from #:to 'objcode #:opts opts
+             #:env (current-module))))
 
 (define (repl-parse repl form)
   (let ((parser (language-parser (repl-language repl))))
     (if parser (parser form) form)))
 
+(define (repl-prepare-eval-thunk repl form)
+  (let* ((eval (language-evaluator (repl-language repl))))
+    (if (and eval
+             (or (null? (language-compilers (repl-language repl)))
+                 (repl-option-ref repl 'interp)))
+        (lambda () (eval form (current-module)))
+        (make-program (repl-compile repl form)))))
+
 (define (repl-eval repl form)
-  (let* ((eval (language-evaluator (repl-language repl)))
-         (thunk (if (and eval
-                         (or (null? (language-compilers (repl-language repl)))
-                             (assq-ref (repl-options repl) 'interp)))
-                    (lambda () (eval form (current-module)))
-                    (make-program (repl-compile repl form '())))))
+  (let ((thunk (repl-prepare-eval-thunk repl form)))
     (% (thunk))))
 
 (define (repl-print repl val)
   (if (not (eq? val *unspecified*))
       (begin
+        (run-hook before-print-hook val)
         ;; The result of an evaluation is representable in scheme, and
         ;; should be printed with the generic printer, `write'. The
         ;; language-printer is something else: it prints expressions of
         ;; a given language, not the result of evaluation.
-	(write val)
-	(newline))))
+	(write val (repl-outport repl))
+	(newline (repl-outport repl)))))
 
 (define (repl-option-ref repl key)
-  (assq-ref (repl-options repl) key))
+  (cadr (or (assq key (repl-options repl))
+            (error "unknown repl option" key))))
 
 (define (repl-option-set! repl key val)
-  (set! (repl-options repl) (assq-set! (repl-options repl) key val)))
+  (let ((spec (or (assq key (repl-options repl))
+                  (error "unknown repl option" key))))
+    (set-car! (cdr spec)
+              (if (procedure? (caddr spec))
+                  ((caddr spec) val)
+                  val))))
+
+(define (repl-default-option-set! key val)
+  (let ((spec (or (assq key repl-default-options)
+                  (error "unknown repl option" key))))
+    (set-car! (cdr spec)
+              (if (procedure? (caddr spec))
+                  ((caddr spec) val)
+                  val))))
+
+(define (repl-default-prompt-set! prompt)
+  (repl-default-option-set! 'prompt prompt))
 
 
 ;;;
