@@ -46,8 +46,9 @@
         (lambda ()
           (let ((ch (next-char #t)))
             (cond ((eof-object? ch)
-                   ;; apparently sometimes even if this is eof, read will
-                   ;; wait on somethingorother. strange.
+                   ;; EOF objects are not buffered. It's quite possible
+                   ;; to peek an EOF then read something else. It's
+                   ;; strange but it's how it works.
                    ch)
                   ((eqv? ch #\,)
                    (read-char port)
@@ -128,7 +129,7 @@
                  (if (eq? k 'quit)
                      (abort args)
                      (begin
-                       (format #t "While executing meta-command:~%" string)
+                       (format #t "While executing meta-command:~%")
                        (pmatch args
                          ((,subr ,msg ,args . ,rest)
                           (display-error #f (current-output-port) subr msg args rest))
@@ -163,7 +164,8 @@
                                     (repl-print repl v))
                                   l))))
                   (lambda (k . args)
-                    (abort args)))))))
+                    (abort args))))
+              #:trap-handler 'disabled)))
            (next-char #f) ;; consume trailing whitespace
            (prompt-loop))))
      (lambda (k status)
